  /*
   *  Object %name    : parser_atp_sst_init_terminate.c%
   *  State           :  %state%
   *  Creation date   :  Apr 13 2009
   *  Last modified   :  %modify_time%
   */
  /** @file
   *  \brief This file includes the DH parsing functions  
   *
   *  \version parser_atp_sst_init_terminate.c#1:csrc:1
   *  \author  taniam
   *  \remarks Copyright (C) 2004 by Discretix Technologies Ltd.
   *           All Rights reserved
   */
/************* Include Files ****************/
#include "SST_SEP_ATP_InitTerminate.h"
#include "parser_atp_init_terminate.h"
#include "SST_SEP_TST.h"
#include "sep_utils.h"
#include "parser_atp_init_terminate.h"
/************************ Global Data ************************/
    //SSTHandle_t                 TST_DataHandle;
    extern DxByte_t*                   TST_Workspace;
    extern DxUint32_t                  TST_WorkspaceSizeInBytes ;
    extern DxUint8_t*                  GlobalMemBuff;
    extern DxUint8_t*                  CurrGlobalMemBuff;
/******************** Private function  **********************/
/*****************************************************************************
* Function Name:
*  DX_SEP_PARSER_SST_InitTerminate_1
*  
* Inputs:
*  msg - message with all requested parameters.
*
* Outputs: 
*  DxUint32_t - Function error return
*
* Description:
* This function does the parsing for the external application SEPQA_SST_Iterator finction.
* It calls the SEP function with parameters parsed from input message
*
* Algorithm: 
* 1. call to DX_SEP_MNG_SSTGetWorkspaceParams
* 2. allocate memory for GlobalMemBuff for all next needed allocation in the test
* 3. allocate buffer from GlobalMemBuff to data struct
* 4. extract parameters from the message 
* 5. call QA SEP function
* 6. free allocated GlobalMemBuff
* 7. Update message with appropritae op code and updated data struct
* 8. end SEP message and return
******************************************************************************/ 
DxUint32_t DX_SEP_PARSER_SST_InitTerminate_1(DxUint32_t* msg)
{
     /********* Variables Definitions ************/
     DxUint32_t    SEP_Error = DX_SUCCESS;

    GlobalMemBuff = DX_NULL;
    CurrGlobalMemBuff = DX_NULL;
    TST_Workspace = DX_NULL;

    /* 1. call to DX_SEP_MNG_SSTGetWorkspaceParams*/
    SEP_Error = DX_SEP_MNG_SSTGetWorkspaceParams(&TST_Workspace,
                                                &TST_WorkspaceSizeInBytes);

    if (TST_Workspace == DX_NULL)
     {
        TST_CheckErrorReturn(SEP_Error,"DX_SEP_MNG_SSTGetWorkspaceParams");
        goto End;
     }

    /*2. allocate memory for GlobalMemBuff for all next needed allocation in the test*/
    GlobalMemBuff = DX_VOS_MemMalloc(sizeof(DxUint8_t)*TST_MAX_SEP_WORKSPACE_MEM);

    if (GlobalMemBuff == DX_NULL)
     {
        TST_CheckErrorReturn(1,"DX_VOS_MemMalloc GlobalMemBuff");
        goto End;
     }
    CurrGlobalMemBuff = GlobalMemBuff;

    /*3. allocate buffer from GlobalMemBuff to data struct*/
    if  ((CurrGlobalMemBuff+sizeof(TST_ATP_DataOpTestDataStruct))>(GlobalMemBuff+TST_MAX_SEP_WORKSPACE_MEM))
    {
        TST_CheckErrorReturn(1,"TST_ATP_DataOpTestData memory allocation");
        goto End;
    }
    else
    {
        TST_ATP_DataOpTestData = (TST_ATP_DataOpTestDataStruct*)CurrGlobalMemBuff;
        CurrGlobalMemBuff = CurrGlobalMemBuff+sizeof(TST_ATP_DataOpTestDataStruct);
    }

    /* 4. extract parameters from the message */
    /******************************************/
    DX_VOS_FastMemCpy(&TST_ATP_DataOpTestData[0], 
                      &msg[DX_SEP_HOST_SEP_BASE_MSG_OFFSET],
                      sizeof(TST_ATP_DataOpTestDataStruct));

    /*5. call QA SEP function */
    /***************************/
    SEP_Error = SEPQA_SST_InitTerminate_1();
    SEP_Error = TST_CheckErrorReturn(SEP_Error,"SEPQA_SST_InitTerminate_1");

End:
    /*6. Free allocated buffers */
    DX_VOS_MemFree(GlobalMemBuff);

    /*7 Update message with appropritae op code and updated data struct*/
    msg[DX_SEP_HOST_SEP_PROTOCOL_MSG_OP_CODE_OFFSET] = DX_SEP_HOST_SEP_PROTOCOL_SST_InitTerminate_1_OP_CODE;

    DX_VOS_FastMemCpy(&msg[DX_SEP_HOST_SEP_PROTOCOL_MSG_RC_OFFSET + 1],
                      &TST_ATP_DataOpTestData[0], 
                      sizeof(TST_ATP_DataOpTestDataStruct));
    /*8. end SEP message and return */
    return DX_SEP_EndMessage(msg, SEP_Error, 4);

}
/*****************************************************************************
* Function Name:
*  DX_SEP_PARSER_SST_InitTerminate_2
*  
* Inputs:
*  msg - message with all requested parameters.
*
* Outputs: 
*  DxUint32_t - Function error return
*
* Description:
* This function does the parsing for the external application SEPQA_SST_InitTerminate finction.
* It calls the SEP function with parameters parsed from input message
*
* Algorithm: 
* 1. call to DX_SEP_MNG_SSTGetWorkspaceParams
* 2. allocate memory for GlobalMemBuff for all next needed allocation in the test
* 3. allocate buffer from GlobalMemBuff to data struct
* 4. extract parameters from the message 
* 5. call QA SEP function
* 6. free allocated GlobalMemBuff
* 7. end SEP message and return
******************************************************************************/ 
DxUint32_t DX_SEP_PARSER_SST_InitTerminate_2(DxUint32_t* msg)
{
     /********* Variables Definitions ************/
     DxUint32_t    SEP_Error = DX_SUCCESS;

    GlobalMemBuff = DX_NULL;
    CurrGlobalMemBuff = DX_NULL;
    TST_Workspace = DX_NULL;

    /* 1. call to DX_SEP_MNG_SSTGetWorkspaceParams*/
    SEP_Error = DX_SEP_MNG_SSTGetWorkspaceParams(&TST_Workspace,
                                                &TST_WorkspaceSizeInBytes);

    if (TST_Workspace == DX_NULL)
     {
        TST_CheckErrorReturn(SEP_Error,"DX_SEP_MNG_SSTGetWorkspaceParams");
        goto End;
     }

    /*2. allocate memory for GlobalMemBuff for all next needed allocation in the test*/
    GlobalMemBuff = DX_VOS_MemMalloc(sizeof(DxUint8_t)*TST_MAX_SEP_WORKSPACE_MEM);

    if (GlobalMemBuff == DX_NULL)
     {
        TST_CheckErrorReturn(1,"DX_VOS_MemMalloc GlobalMemBuff");
        goto End;
     }
    CurrGlobalMemBuff = GlobalMemBuff;

    /*3. allocate buffer from GlobalMemBuff to data struct*/
    if  ((CurrGlobalMemBuff+sizeof(TST_ATP_DataOpTestDataStruct))>(GlobalMemBuff+TST_MAX_SEP_WORKSPACE_MEM))
    {
        TST_CheckErrorReturn(1,"TST_ATP_DataOpTestData memory allocation");
        goto End;
    }
    else
    {
        TST_ATP_DataOpTestData = (TST_ATP_DataOpTestDataStruct*)CurrGlobalMemBuff;
        CurrGlobalMemBuff = CurrGlobalMemBuff+sizeof(TST_ATP_DataOpTestDataStruct);
    }

    /* 4. extract parameters from the message */
    /******************************************/
    DX_VOS_FastMemCpy(&TST_ATP_DataOpTestData[0], 
                      &msg[DX_SEP_HOST_SEP_BASE_MSG_OFFSET],
                      sizeof(TST_ATP_DataOpTestDataStruct));

    /*5. call QA SEP function */
    /***************************/
    SEP_Error = SEPQA_SST_InitTerminate_2();
    SEP_Error = TST_CheckErrorReturn(SEP_Error,"SEPQA_SST_InitTerminate_2");

End:
    /*6. Free allocated buffers */
    DX_VOS_MemFree(GlobalMemBuff);

    /*7. end SEP message and return */
    msg[DX_SEP_HOST_SEP_PROTOCOL_MSG_OP_CODE_OFFSET] = DX_SEP_HOST_SEP_PROTOCOL_SST_InitTerminate_2_OP_CODE;
    return DX_SEP_EndMessage(msg, SEP_Error, 4);

}
